from django.shortcuts import render, redirect
from django.http import JsonResponse
from django.conf import settings
import time
import os
import rust_crud_api # Custom Rust Pyo3 Library 🦀

db_url = settings.DATABASE_URL

def register_view(request):
    """
    Handles user registration. Expects POST with
    'name', 'email', 'password', 'username', 'studentid', 'startyear', 'endyear',
    and optionally 'profilepicture'.
    """
    db_url = settings.DATABASE_URL  # Make sure DATABASE_URL is defined in your Django settings
    context = {}

    if request.method == 'POST':
        name = request.POST.get('name', '').strip()
        email = request.POST.get('email', '').strip()
        password = request.POST.get('password', '').strip()
        username = request.POST.get('username', '').strip()
        studentid_str = request.POST.get('studentid', '').strip()
        startyear_str = request.POST.get('startyear', '').strip()
        endyear_str = request.POST.get('endyear', '').strip()

        # 1. Check required fields
        if not all([name, email, password, username, studentid_str, startyear_str, endyear_str]):
            context['error'] = (
                "Name, email, username, student ID, start year, and end year are required."
            )
            return render(request, 'myapp/register.html', context)

        # 2. Try converting studentid, startyear, endyear to int
        try:
            studentid = int(studentid_str)
        except ValueError:
            context['error'] = "Student ID must be a number."
            return render(request, 'myapp/register.html', context)

        try:
            startyear = int(startyear_str)
        except ValueError:
            context['error'] = "Start year must be a number."
            return render(request, 'myapp/register.html', context)

        try:
            endyear = int(endyear_str)
        except ValueError:
            context['error'] = "End year must be a number."
            return render(request, 'myapp/register.html', context)

        # 3. Handle profile picture upload (if any)
        profilepicture_file = request.FILES.get('profilepicture')
        saved_path = None  # Default to None if no file is uploaded

        if profilepicture_file:
            # Make sure your form <input> matches name="profilepicture"
            filename = profilepicture_file.name

            # Decide a directory inside MEDIA_ROOT for profile pics
            profile_dir = os.path.join(settings.MEDIA_ROOT, 'profile_pics')
            os.makedirs(profile_dir, exist_ok=True)  # Ensure the folder exists

            # Build the full filesystem path where we’ll store the file
            save_path = os.path.join(profile_dir, filename)

            # Write the file to disk in chunks
            with open(save_path, 'wb+') as destination:
                for chunk in profilepicture_file.chunks():
                    destination.write(chunk)

            # The path we'll store in the DB (so we can serve it at /media/profile_pics/<filename>)
            saved_path = f"/media/profile_pics/{filename}"

        # Debug prints to confirm we have the file and path
        print("DEBUG: profilepicture_file is:", profilepicture_file)
        print("DEBUG: saved_path is:", saved_path)

        # 4. Call Rust to create the user
        try:
            rust_crud_api.create_user(
                db_url,
                name,
                email,
                password,
                username,
                studentid,
                startyear,
                endyear,
                saved_path
            )
            # 5. After successful registration, redirect to login or some other page
            return redirect('login')
        except Exception as e:
            context['error'] = f"An error occurred: {str(e)}"
            return render(request, 'myapp/register.html', context)
    
    # If GET or anything else, just render the registration form
    return render(request, 'myapp/register.html', context)



def account_view(request):
    """
    Display the account page for the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    try:
        user = rust_crud_api.get_user(db_url, int(user_id))
        if user is None:
            return JsonResponse({'error': 'User not found.'}, status=404)
        return render(request, 'myapp/account.html', {'user': user})
    except Exception as e:
        return JsonResponse({'error': str(e)}, status=500)


def delete_account_view(request):
    """
    Delete the account of the logged-in user.
    """
    user_id = request.session.get('user_id')
    if not user_id:
        return redirect('login')
    db_url = settings.DATABASE_URL
    if request.method == 'POST':
        try:
            success = rust_crud_api.delete_user(db_url, int(user_id))
            if success:
                request.session.flush()  # Clear the session after deletion.
                return redirect('register')
            else:
                return JsonResponse({'error': 'Account deletion failed.'}, status=500)
        except Exception as e:
            return JsonResponse({'error': str(e)}, status=500)
    return render(request, 'myapp/delete_account.html')